@@ -1,6 +1,6 @@
 /*  TimelineView.cxx
  *
- *  Copyright (C) 2005 Richard Spindler <richard.spindler AT gmail.com>
+ *  Copyright (C) 2005-2007 Richard Spindler <richard.spindler AT gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <cassert>
 
 
 #include <FL/Fl.H>
@@ -52,11 +53,13 @@
 #include "IPlaybackCore.H"
 #include "DocManager.H"
 #include "Commands.H"
-#include "Frei0rFactoryPlugin.H"
-#include "Frei0rEffect.H"
-#include "Frei0rDialog.H"
+#include "IVideoEffect.H"
+#include "IEffectDialog.H"
 #include "PasteSelectionCommand.H"
 #include "TitleClip.H"
+//#include "HistogramView.H"
+#include "AudioVolumeFilterFactory.H" //TODO: Remove these two includes
+#include "ColorCurveFactory.H"
 
 #include "audio.xpm"
 #include "video.xpm"
@@ -86,10 +89,18 @@ TimelineView::TimelineView( int x, int y, int w, int, const char *label )
 TimelineView::~TimelineView()
 {
 }
+struct action_menu_struct {
+	Clip* clip;
+	int index;
+};
+void action_menu_callback( Fl_Widget*, void* data ) {
+	struct action_menu_struct* tupel = (struct action_menu_struct*)data;
+	tupel->clip->doAction( tupel->index );
+}
 #define MENU_ITEM_INIT 0, 0, 0, 0, 0, 0, 0, 0
 int TimelineView::handle( int event )
 {
-  Clip* cl;
+	Clip* cl;
 	if ( g_playbackCore->active() ) {
 		return Fl_Widget::handle( event );
 	}
@@ -102,8 +113,25 @@ int TimelineView::handle( int event )
 				char *fn,*filename=strdup(Fl::event_text());
 				int i=strlen(filename);
 				while (i>0 && (iscntrl(filename[i]) || isspace(filename[i])) ) filename[i--]=0;
-				if (!strncmp(filename,"file://",7)) fn=&(filename[7]); 
-				else fn=filename;
+				if ( !strncmp( filename, "filter:", 7 ) || !strncmp( filename, "effect:", 7 ) ) {
+					cl = get_clip( _x, _y );
+					FilterClip* fc = dynamic_cast<FilterClip*>(cl);
+					if ( fc ) {
+						clear_selection();
+						Command* cmd = new FilterAddCommand( fc, filename );
+						submit( cmd );
+						g_videoView->redraw();
+						toggle_selection( fc );
+						//TODO: Display Dialog
+					}
+					free(filename);
+					return 1;
+				}
+				if (!strncmp(filename,"file://",7)) {
+					fn=&(filename[7]); 
+				} else {
+					fn=filename;
+				}
 				if (t && !fl_filename_isdir(fn)) {
 					int64_t rp = get_real_position( _x, t->stretchFactor() );
 					
@@ -156,11 +184,6 @@ int TimelineView::handle( int event )
 						int yy = get_track_top( i->track );
 						if ( _y > yy && _y < yy + i->track->h() ) {
 							Fl_Menu_Item menuitem[] = { { "Remove Track", MENU_ITEM_INIT }, { "Move Up", MENU_ITEM_INIT }, { "Move Down", MENU_ITEM_INIT }, { "Rename", 0,0,0,FL_MENU_DIVIDER,0,0,0,0 }, { "1x", MENU_ITEM_INIT }, { "2x", MENU_ITEM_INIT }, { "4x", MENU_ITEM_INIT }, { 0L, MENU_ITEM_INIT } };
-							if ( i->track->type() != TRACK_TYPE_AUDIO ) {
-								menuitem[3].flags = 0;
-								menuitem[4] = menuitem[7];
-							}
-							
 							Fl_Menu_Item* r = (Fl_Menu_Item*)menuitem->popup( TRACK_SPACING + x(), yy + i->track->h() + y() + 1 );
 							if ( r == &menuitem[0] ) {
 								clear_selection();
@@ -178,15 +201,18 @@ int TimelineView::handle( int event )
 									i->track->name( name );
 									redraw();
 								}
-							} else if ( i->track->type() == TRACK_TYPE_AUDIO && r == &menuitem[4] ) {
+							} else if ( r == &menuitem[4] ) {
 								i->track->h( 30 );
 								redraw();
 							} else if ( i->track->type() == TRACK_TYPE_AUDIO && r == &menuitem[5] ) {
 								i->track->h( 60 );
 								redraw();
-							} else if ( i->track->type() == TRACK_TYPE_AUDIO && r == &menuitem[6] ) {
+							} else if ( r == &menuitem[6] ) {
 								i->track->h( 120 );
 								redraw();
+							} else if ( i->track->type() == TRACK_TYPE_VIDEO && r == &menuitem[5] ) {
+								i->track->h( 65 );
+								redraw();
 							}
 							return 1;
 						}
@@ -210,7 +236,8 @@ int TimelineView::handle( int event )
 							{ "Fit", 0,0,0,FL_MENU_RADIO|( vcl->fit() ? FL_MENU_VALUE : 0 ),0,0,0,0 },
 							{ "Stretch", 0,0,0,FL_MENU_RADIO |( vcl->stretch() ? FL_MENU_VALUE : 0 ),0,0,0,0 },
 							{ "Default", 0,0,0,FL_MENU_RADIO | FL_MENU_DIVIDER|( vcl->def() ? FL_MENU_VALUE : 0 ),0,0,0,0 }, 
-							{ "Unmute Original Sound", MENU_ITEM_INIT }, { "Select all Clips after Cursor", MENU_ITEM_INIT }, 
+							{ "Unmute Original Sound", MENU_ITEM_INIT },
+							{ "Select all Clips after Cursor", MENU_ITEM_INIT }, 
 							{ 0L, MENU_ITEM_INIT } };
 						//menuitem[2].setonly();
 						Fl_Menu_Item* r = (Fl_Menu_Item*)menuitem->popup( Fl::event_x(), Fl::event_y() );
@@ -267,30 +294,53 @@ int TimelineView::handle( int event )
 				}
 				VideoEffectClip* vec;
 				if ( ( vec = dynamic_cast<VideoEffectClip*>(cl) ) && ( Fl::event_button() == FL_RIGHT_MOUSE ) ) {
-						Fl_Menu_Item menuitem[] = { { "Crop", 0,0,0,FL_MENU_RADIO|( vec->crop() ? FL_MENU_VALUE : 0 ),0,0,0,0 },
-							{ "Fit", 0,0,0,FL_MENU_RADIO|( vec->fit() ? FL_MENU_VALUE : 0 ),0,0,0,0 },
-							{ "Stretch", 0,0,0,FL_MENU_RADIO |( vec->stretch() ? FL_MENU_VALUE : 0 ),0,0,0,0 },
-							{ "Default", 0,0,0,FL_MENU_RADIO | FL_MENU_DIVIDER|( vec->def() ? FL_MENU_VALUE : 0 ),0,0,0,0 }, 
-							{ "Select all Clips after Cursor", MENU_ITEM_INIT }, 
-							{ 0L, MENU_ITEM_INIT } };
-						//menuitem[2].setonly();
-						Fl_Menu_Item* r = (Fl_Menu_Item*)menuitem->popup( Fl::event_x(), Fl::event_y() );
-						if ( r == &menuitem[0] ) {
-							vec->crop( true );
-							g_videoView->redraw();
-						} else if ( r == &menuitem[1] ) {
-							vec->fit( true );
-							g_videoView->redraw();
-						} else if ( r == &menuitem[2] ) {
-							vec->stretch( true );
-							g_videoView->redraw();
-						} else if ( r == &menuitem[3] ) {
-							vec->def( true );
-							g_videoView->redraw();
-						} else if ( r == &menuitem[4] ) {
-							select_all_after_cursor();	
-						}
-						return 1;
+					struct action_menu_struct action_menu_tuples[cl->getActionCount()];
+					Fl_Menu_Item action_menu[cl->getActionCount()+1];
+					//Fl_Menu_ action_menu(0,0,0,0);
+					for ( int i = 0; i < cl->getActionCount(); i++ ) {
+						action_menu_tuples[i].clip = cl;
+						action_menu_tuples[i].index = i;
+						action_menu[i].text = cl->getActionName( i );
+						action_menu[i].shortcut_ = 0;
+						action_menu[i].callback_ = (Fl_Callback*)action_menu_callback;
+						action_menu[i].user_data_ = &(action_menu_tuples[i]);
+						action_menu[i].flags = 0;
+						action_menu[i].labeltype_ = FL_NORMAL_LABEL;
+						action_menu[i].labelfont_ = FL_HELVETICA;
+						action_menu[i].labelsize_ = 14;
+						action_menu[i].labelcolor_ = FL_BLACK;
+					}
+					action_menu[cl->getActionCount()].text = 0;
+
+					Fl_Menu_Item menuitem[] = { { "Crop", 0,0,0,FL_MENU_RADIO|( vec->crop() ? FL_MENU_VALUE : 0 ),0,0,0,0 },
+						{ "Fit", 0,0,0,FL_MENU_RADIO|( vec->fit() ? FL_MENU_VALUE : 0 ),0,0,0,0 },
+						{ "Stretch", 0,0,0,FL_MENU_RADIO |( vec->stretch() ? FL_MENU_VALUE : 0 ),0,0,0,0 },
+						{ "Default", 0,0,0,FL_MENU_RADIO | FL_MENU_DIVIDER|( vec->def() ? FL_MENU_VALUE : 0 ),0,0,0,0 }, 
+						{ "Actions", 0,0,action_menu,FL_SUBMENU_POINTER,0,0,0,0},
+						{ "Select all Clips after Cursor", MENU_ITEM_INIT }, 
+						{ 0L, MENU_ITEM_INIT } };
+					//menuitem[2].setonly();
+					Fl_Menu_Item* r = (Fl_Menu_Item*)menuitem->popup( Fl::event_x(), Fl::event_y() );
+					if ( r && r->callback() ) {
+						 Fl_Callback* cb = r->callback();
+						 cb( 0, r->user_data() );
+					}
+					if ( r == &menuitem[0] ) {
+						vec->crop( true );
+						g_videoView->redraw();
+					} else if ( r == &menuitem[1] ) {
+						vec->fit( true );
+						g_videoView->redraw();
+					} else if ( r == &menuitem[2] ) {
+						vec->stretch( true );
+						g_videoView->redraw();
+					} else if ( r == &menuitem[3] ) {
+						vec->def( true );
+						g_videoView->redraw();
+					} else if ( r == &menuitem[4] ) {
+						select_all_after_cursor();	
+					}
+					return 1;
 				}
 				if ( Fl::event_button() == FL_RIGHT_MOUSE ) {
 					Fl_Menu_Item menuitem[] = { { "Select all Clips after Cursor", MENU_ITEM_INIT }, { 0L, MENU_ITEM_INIT } };
@@ -299,12 +349,15 @@ int TimelineView::handle( int event )
 					}
 					return 1;
 				}
-				if ( cl && g_ui->automationsMode() == 1 && cl->has_automation() ) {
-					if ( FL_SHIFT & Fl::event_state() ) {
+				AudioClip* audioClip = dynamic_cast<AudioClip*>(cl);
+				if ( cl && g_ui->automationsMode() == 1 && audioClip ) {
+					Rect r = get_clip_rect( cl, true );
+					m_dragHandler = audioClip->onMouseDown( r, _x, _y, FL_SHIFT & Fl::event_state() );
+					/*if ( FL_SHIFT & Fl::event_state() ) {
 						m_dragHandler = new ShiftAutomationDragHandler( cl, get_clip_rect( cl, true ), _x, _y );
 					} else {
 						m_dragHandler = new AutomationDragHandler( cl, get_clip_rect( cl, true ), 0, _x, _y );
-					}
+					}*/
 					return 1;
 				} else if ( cl ) {
 					if ( _x < get_screen_position( cl->position(), cl->track()->stretchFactor() ) + 8 ) {
@@ -350,6 +403,21 @@ int TimelineView::handle( int event )
 					return 1;
 				}
 			}
+		case FL_SHORTCUT: //patch by raaf from OME Forums
+			{
+				int key = Fl::event_key();
+				if(key == FL_KP + '4')
+					move_cursor(m_stylusPosition - 1000000);
+				else if(key ==  FL_KP + '6')
+					move_cursor(m_stylusPosition + 1000000);
+				else if(key == FL_KP + '2')
+					move_cursor(m_stylusPosition - 50000000);
+				else if(key == FL_KP + '8')
+					move_cursor(m_stylusPosition + 50000000);
+				else
+					return Fl_Widget::handle( event );
+			}
+			return 1;
 		default:
 			return Fl_Widget::handle( event );	
 	}
@@ -445,43 +513,6 @@ void TimelineView::draw()
 			} else {
 				fl_draw_box( FL_BORDER_FRAME, scr_clip_x, scr_clip_y, scr_clip_w, scr_clip_h, FL_DARK3 );
 			}
-			if ( track->type() == TRACK_TYPE_AUDIO ) {
-				//Draw Automations
-				AudioClip* audioClip = dynamic_cast<AudioClip*>(clip);
-
-				if ( !audioClip ) {
-					continue;
-				}
-				fl_push_clip( scr_clip_x, scr_clip_y, scr_clip_w, scr_clip_h );
-				
-				auto_node* nodes = audioClip->getAutoPoints();
-
-				fl_color( FL_RED );
-				for ( ; nodes && nodes->next; nodes = nodes->next ) {
-					int y = (int)( scr_clip_y + ( ( track->h() - 10 ) * ( 1.0 - nodes->y ) ) + 5 );
-					int y_next = (int)( scr_clip_y + ( ( track->h() - 10 ) * ( 1.0 - nodes->next->y ) ) + 5 );
-					fl_line( get_screen_position( audioClip->position() + nodes->x, track->stretchFactor() ),
-							y,
-							get_screen_position( audioClip->position() + nodes->next->x, track->stretchFactor() ),
-							y_next );
-				}
-				nodes = audioClip->getAutoPoints();
-				for ( ; nodes; nodes = nodes->next ) {
-					//consider Trimming
-					int x;
-					int y = (int)( scr_clip_y + ( ( track->h() - 10 ) * ( 1.0 - nodes->y ) ) );
-					if ( !nodes->next ) {
-						x = get_screen_position( audioClip->position() + nodes->x, track->stretchFactor() ) - 10;
-					} else if ( nodes == audioClip->getAutoPoints() ) {
-						x = get_screen_position( audioClip->position() + nodes->x, track->stretchFactor() );
-					} else {
-						x = get_screen_position( audioClip->position() + nodes->x, track->stretchFactor() ) - 5;
-					}
-					fl_draw_box( FL_UP_BOX, x, y, 10, 10, FL_RED );
-				}
-				fl_pop_clip();
-			}
-
 		}
 		if ( dynamic_cast<VideoTrack*>(track) ) {
 			for ( clip_node* j = track->getClips(); j; j = j->next ) {
@@ -555,7 +586,7 @@ int64_t TimelineView::get_real_position( int p, float stretchFactor )
 }
 int64_t TimelineView::get_screen_position( int64_t p, float stretchFactor )
 {
-	return llrint( float( p - ( m_scrollPosition * stretchFactor ) ) * SwitchBoard::i()->zoom() / stretchFactor ) + LEFT_TRACK_SPACING + x();
+	return llrint( float( float(p) - ( m_scrollPosition * stretchFactor ) ) * SwitchBoard::i()->zoom() / stretchFactor ) + LEFT_TRACK_SPACING + x();
 
 }
 void TimelineView::scroll( int64_t position )
@@ -564,7 +595,7 @@ void TimelineView::scroll( int64_t position )
 }
 void TimelineView::zoom( float zoom )
 {
-	if ( isinf(zoom) || isnan(zoom) || zoom >= 10.0 ) {
+	if ( isinf(zoom) || isnan(zoom) || zoom >= 300.0 ) {
 		redraw();
 		e_stylus_position( get_screen_position(m_stylusPosition) );
 		return;
@@ -786,7 +817,7 @@ void TimelineView::select_all_after_cursor()
 	clear_selection();
 	for ( track_node* o = g_timeline->getTracks(); o; o = o->next ) {
 		for ( clip_node* c = o->track->getClips(); c; c = c->next ) {
-			if ( c->clip->position() / c->clip->track()->stretchFactor() >= m_stylusPosition ) {
+			if ( c->clip->position() / c->clip->track()->stretchFactor() * NLE_TIME_BASE >= m_stylusPosition ) {
 				clip_node* n = new clip_node;
 				n->next = 0;
 				n->clip = c->clip;
@@ -901,16 +932,16 @@ void TimelineView::updateEffectDisplay()
 		g_ui->m_effectMenu->deactivate();
 		return;
 	}
-	VideoEffectClip* vc = dynamic_cast<VideoEffectClip*>( m_selectedClips->clip );
+	FilterClip* vc = dynamic_cast<FilterClip*>( m_selectedClips->clip );
 	if ( !vc ) {
 		g_ui->m_effectMenu->deactivate();
 		return;
 	}
 	g_ui->m_effectMenu->activate();
 	// Liste füllen
-	effect_stack* es = vc->getEffects();
+	filter_stack* es = vc->getFilters();
 	while ( es ) {
-		g_ui->effect_browser->add( es->effect->name(), es->effect );
+		g_ui->effect_browser->add( es->filter->name(), es->filter );
 		es = es->next;
 	}
 }
@@ -1000,7 +1031,14 @@ void TimelineView::moveEffectUp()
 	g_ui->setEffectButtons();
 	g_videoView->redraw();
 }
-void TimelineView::addEffect( AbstractEffectFactory* effectFactory )
+
+/*
+static FilterFactory* findFilterFactory( const char* name )
+{
+}
+*/
+
+void TimelineView::addEffect( FilterFactory* effectFactory )
 {
 	if ( !m_selectedClips ) {
 		return;
@@ -1008,14 +1046,25 @@ void TimelineView::addEffect( AbstractEffectFactory* effectFactory )
 	if ( m_selectedClips->next ) {
 		return;
 	}
-	VideoEffectClip* vc = dynamic_cast<VideoEffectClip*>( m_selectedClips->clip );
-	if ( !vc ) {
+	FilterClip* fc = dynamic_cast<FilterClip*>( m_selectedClips->clip );
+	if ( !fc ) {
 		return;
 	}
-	vc->pushEffect( effectFactory );
+	FilterAddCommand* cmd = new FilterAddCommand( fc, effectFactory->identifier() );
+	submit( cmd );
+	
+	FilterBase* fe = cmd->m_filter;
+	assert(fe);
 	updateEffectDisplay();
 	g_ui->setEffectButtons();
 	g_videoView->redraw();
+	IEffectDialog* dialog = fe->dialog();
+	//TODO: VideoEffects: move Stylus to start of clip if it is not inside the clip.
+
+	IVideoEffect* ive = dynamic_cast<IVideoEffect*>(fe);
+	if ( ive && ive->numParams() && dialog ) {
+		dialog->show();
+	}
 }
 void TimelineView::removeEffect()
 {
@@ -1025,15 +1074,16 @@ void TimelineView::removeEffect()
 	if ( m_selectedClips->next ) {
 		return;
 	}
-	VideoEffectClip* vc = dynamic_cast<VideoEffectClip*>( m_selectedClips->clip );
-	if ( !vc ) {
+	FilterClip* fc = dynamic_cast<FilterClip*>( m_selectedClips->clip );
+	if ( !fc ) {
 		return;
 	}
 	int c = g_ui->effect_browser->value();
 	if ( c == 0 ) {
 		return;
 	}
-	vc->removeEffect( c );
+	Command* cmd = new FilterRemoveCommand( fc, c );
+	submit( cmd );
 	updateEffectDisplay();
 	g_ui->setEffectButtons();
 	g_videoView->redraw();
@@ -1046,7 +1096,7 @@ void TimelineView::editEffect()
 	if ( m_selectedClips->next ) {
 		return;
 	}
-	VideoEffectClip* vc = dynamic_cast<VideoEffectClip*>( m_selectedClips->clip );
+	FilterClip* vc = dynamic_cast<FilterClip*>( m_selectedClips->clip );
 	if ( !vc ) {
 		return;
 	}
@@ -1054,29 +1104,16 @@ void TimelineView::editEffect()
 	if ( c == 0 ) {
 		return;
 	}
-	effect_stack* es = vc->getEffects();
+	filter_stack* es = vc->getFilters();
 	for ( int i = 1; i < c && es ; i++ ) {
 		es = es->next;
 	}
-/*	Frei0rEffect* fe = dynamic_cast<Frei0rEffect*>( vc->getEffect() );
-	for ( int i = 1; i < c && fe ; i++ ) {
-		fe = dynamic_cast<Frei0rEffect*>( fe->getReader() );
-	}*/
 	if ( !es ) {
 		return;
 	}
-	Frei0rEffect* fe = dynamic_cast<Frei0rEffect*>( es->effect );
-	Frei0rDialog* dialog = dynamic_cast<Frei0rDialog*>(fe->m_dialog);
-	if ( !dialog ) {
-		dialog = new Frei0rDialog( fe );
-		fe->m_dialog = dialog;
-	}
+	FilterBase* fe = es->filter;
+	IEffectDialog* dialog = fe->dialog();
 	dialog->show();
-/*	Frei0rDialog dialog( fe );
-	dialog.show();
-	while ( dialog.shown() ) {
-		Fl::wait();
-	}*/
 
 }
 
@@ -1084,14 +1121,6 @@ void TimelineView::trim_clip( Clip* clip, int _x, bool trimRight )
 {
 	Command* cmd = new TrimCommand( clip, get_real_position( _x, clip->track()->stretchFactor() ), trimRight );
 	g_docManager->submit( cmd );
-	/*
-	if (trimRight) {
-		int64_t trimv = int64_t( ( clip->position() + clip->length() ) - ( get_real_position(_x, clip->track()->stretchFactor()) ) );
-		clip->trimB( trimv );
-		adjustScrollbar();
-		return;
-	}
-	clip->trimA( int64_t( get_real_position(_x, clip->track()->stretchFactor())  - clip->position() ) );*/
 	adjustScrollbar();
 }
 void TimelineView::move_cursor( int64_t position )
@@ -1102,20 +1131,30 @@ void TimelineView::move_cursor( int64_t position )
 	}
 	window()->make_current();
 	long screen_pos = get_screen_position(m_stylusPosition);
-	if ( screen_pos < ( LEFT_TRACK_SPACING + x() ) ) {
-		m_scrollPosition = get_real_position( screen_pos );
+	if ( screen_pos > w() + x() - 30 ) {
+		m_scrollPosition += (int64_t)( ( 30 - x() - w() + screen_pos ) / SwitchBoard::i()->zoom() );
 		adjustScrollbar();
 		redraw();
-	} else if ( screen_pos > w() + x() - TRACK_SPACING - 50 ) {
-		m_scrollPosition = get_real_position( screen_pos );
+	} else if ( screen_pos < x() + LEFT_TRACK_SPACING + 20 && m_scrollPosition > 0 ) {
+		m_scrollPosition -= (int64_t)( ( 20 - ( screen_pos - x() - LEFT_TRACK_SPACING ) ) / SwitchBoard::i()->zoom() );
+		if ( m_scrollPosition < 0 ) { m_scrollPosition = 0; }
 		adjustScrollbar();
 		redraw();
 	} else {
-		window()->make_current();
 		fl_overlay_rect( get_screen_position(m_stylusPosition), parent()->y(), 1, parent()->h() );
 	}
 	e_stylus_position( get_screen_position(m_stylusPosition) );
 	e_seek_position( m_stylusPosition );
+
+	// signal histogram view
+/*	if ( m_selectedClips && !m_selectedClips->next ) {
+		VideoEffectClip* vc = dynamic_cast<VideoEffectClip*>( m_selectedClips->clip );
+		Clip* clip = m_selectedClips->clip;
+		if ( vc && m_stylusPosition >= clip->A() && m_stylusPosition <= clip->B() ) {
+			g_histogram->setVideoClip( vc, m_stylusPosition );
+		}
+
+	}*/
 }
 void TimelineView::stylus( long stylus_pos )
 {
